package com.cultural.heritage.controller.good;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.cart.CartRecordAddRequest;
import com.cultural.heritage.model.dto.cart.CartRecordUpdateRequest;
import com.cultural.heritage.model.entity.CartRecord;
import com.cultural.heritage.model.vo.CartRecordVO;
import com.cultural.heritage.service.good.CartRecordService;
import com.cultural.heritage.service.good.GoodService;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/cart")
@Slf4j
@Tag(name = "购物车接口")
public class CartRecordController {


    @Resource
    private CartRecordService cartRecordService;



    /**
     * 用户添加商品至购物车
     * @param cartRecordAddRequest  购物车记录添加请求体
     * @return  是否添加成功
     */
    @PostMapping("/add")
    public BaseResponse<Boolean> addCartRecord(@RequestBody CartRecordAddRequest cartRecordAddRequest) {
        if (cartRecordAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        boolean result = cartRecordService.dealAlreadyExistGood(cartRecordAddRequest);
        return ResultUtils.success(result);
    }


    /**
     * 用户更新商品至购物车
     * @param cartRecordUpdateRequestList   购物车记录列表更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update")
    public BaseResponse<Boolean> updateCartRecord(@RequestBody List<CartRecordUpdateRequest> cartRecordUpdateRequestList) {
        if (CollectionUtils.isEmpty(cartRecordUpdateRequestList)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        List<CartRecord> cartRecordList = cartRecordUpdateRequestList.stream().map(cartRecordUpdateRequest -> {
            CartRecord cartRecord = new CartRecord();
            BeanUtils.copyProperties(cartRecordUpdateRequest, cartRecord);
            return cartRecord;
        }).toList();
        boolean result = cartRecordService.updateBatchById(cartRecordList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }


    /**
     * 用户删除购物车的物品
     * @param idList  删除的商品id列表
     * @return 是否删除成功
     */
    @PostMapping("/delete")
    public BaseResponse<Boolean> deleteCart(@RequestBody List<Integer> idList) {
        if (CollectionUtils.isEmpty(idList)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        boolean result = cartRecordService.removeBatchByIds(idList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }


    /**
     * 展示用户购物车中的商品
     * @param userIdRequest  用户id请求体
     * @return
     */
    @PostMapping("/list")
    public BaseResponse<List<CartRecordVO>> listUserCartRecord(@RequestBody CommonRequest userIdRequest) {
        if (userIdRequest == null || userIdRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 根据用户id查询购物车记录
        Long userId = userIdRequest.getId();
        QueryWrapper<CartRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId);
        List<CartRecord> list = cartRecordService.list(queryWrapper);
        List<CartRecordVO> cartRecordVOList = cartRecordService.transformToCartRecordVOList(list);
        return ResultUtils.success(cartRecordVOList);
    }
}
