package com.cultural.heritage.controller.good;


import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.category.CategoryAddRequest;
import com.cultural.heritage.model.entity.Category;
import com.cultural.heritage.model.entity.Good;
import com.cultural.heritage.service.good.CategoryService;
import com.cultural.heritage.service.good.GoodService;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/category")
@Slf4j
@Tag(name = "商品类别接口")
public class CategoryController {

    @Resource
    private CategoryService categoryService;



    @Resource
    private GoodService goodService;



    /**
     * 添加商品类别
     * @param categoryAddRequest  类别添加请求体
     * @return  是否添加成功
     */
    @PostMapping("/add")
    public BaseResponse<Boolean> addCategory(@RequestBody CategoryAddRequest categoryAddRequest) {
        if (categoryAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Category category = new Category();
        BeanUtils.copyProperties(categoryAddRequest, category);
        categoryService.validCategory(category, true);
        boolean save = categoryService.save(category);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true, "类别插入成功");
    }


    /**
     * 删除商品类别
     * @param deleteCategoryRequest 类别删除请求体
     * @return  是否删除成功
     */
    @PostMapping("/delete")
    public BaseResponse<Boolean> deleteCategory(@RequestBody CommonRequest deleteCategoryRequest) {
        if (deleteCategoryRequest == null || deleteCategoryRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = deleteCategoryRequest.getId();
        boolean result = categoryService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true, "类别删除成功");
    }



    /**
     * 更新商品类别
     * @param category  类别更新请求体
     * @return 是否更新成功
     */
    @PostMapping("/update")
    public BaseResponse<Boolean> updateCategory(@RequestBody Category category) {
        if (category == null || category.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        categoryService.validCategory(category, false);
        boolean result = categoryService.updateById(category);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true, "类别更新成功");
    }


    /**
     * 查询所有商品类别
     * @return  商品类别列表
     */
    @PostMapping("/list")
    public BaseResponse<List<Category>> listCategory() {
        List<Category> list = categoryService.list();
        return ResultUtils.success(list);
    }



    /**
     * 根据类别查询商品列表
     * @param categoryQueryRequest  类别查询请求体
     * @return 当前类别的商品列表
     */
    @PostMapping("/list/type")
    public BaseResponse<Map<Category, List<Good>>> listGoodByCategory(@RequestBody CommonRequest categoryQueryRequest) {
        if (categoryQueryRequest == null || categoryQueryRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = categoryQueryRequest.getId();
        Category category = categoryService.getCategoryById(id);
        String typeName = category.getTypeName();
        if (StringUtils.isBlank(typeName)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        List<Good> goodList = goodService.getGoodListByTypeName(typeName);
        Map<Category, List<Good>> map = new HashMap<>();
        map.put(category, goodList);
        return ResultUtils.success(map);
    }



}
